@@ -15,10 +15,12 @@ WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License."""
 import ast
+import datetime
 import enum
 import json
 from binascii import hexlify
 
+import validators
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
@@ -37,19 +39,27 @@ from .cert_engine import CertDnsEngine
 
 class Cert(StoreObject):
     """Object representation of a TLS certificate"""
-    _body: str  #: String representation of private, chain and public portions of certificate as a map/json
-    _info: str  #: Certificate details
-    _data: {}  #: Combined body and info map
-    _file: object
-    _x509: x509
-    _common_name: str
-    _jinja: Environment
+    _body: str           #: String representation of private, chain and public portions of certificate as a map/json
+    _info: str           #: Certificate details
+    _data: {}            #: Combined body and info map
+    _path: str           #: Objects stored using <mount><path><name><type>
+    _policy: str         #: Vault access policy, gen from jinja template, explicit to instance of cert
+    _mount: str          #: Based on certificate its mount is either KNOX_VAULT_MOUNT or KNOX_VAULT_MOUNT/client
+    _file: object        #: Raw file contents of certificate
+    _x509: x509          #: Parsed data object from raw file
+    _common_name: str    #: Defaults to value from certificate
+    _type: str           #: Certificate type identifier
+    _jinja: Environment  #: Template engine
 
     class CertTypes(enum.Enum):
         PEM = 1
         DER = 2
         PFX = 3
 
+        @classmethod
+        def valid(cls, name) -> bool:
+            return any(x for x in cls if x.name == name)
+
     PEM = CertTypes.PEM
     DER = CertTypes.DER
     PFX = CertTypes.PFX
@@ -60,11 +70,13 @@ class Cert(StoreObject):
         self._common_name = self.valid_name(common_name)
         self._body = ""
         self._info = ""
-        super().__init__(name=self._common_name, path=self.store_path(), body=self._body, info=self._info)
+        self._type = ""
+        super().__init__(name=self.name, path=self.path, body=self._body, info=self._info)
         self._jinja = Environment(loader=FileSystemLoader('templates'))
         self._tmpl_body = self._jinja.get_template('body_template.js')
         self._tmpl_info = self._jinja.get_template('info_template.js')
         self._tmpl_data = self._jinja.get_template('data_template.js')
+        self._tmpl_policy = self._jinja.get_template('policy_template.js')
 
     def load_x509(self, path: str) -> None:
         """Given path to PEM x509 read in certificate
@@ -72,6 +84,8 @@ class Cert(StoreObject):
             :param path: File path to x509 PEM file
             :type path: str
         """
+        self.type = Cert.PEM.name
+
         with open(path, mode='r+', encoding='utf-8') as fp:
             self._file = fp.read()
             self._x509 = x509.load_pem_x509_certificate(bytes(self._file, 'utf-8'), default_backend())
@@ -82,19 +96,21 @@ class Cert(StoreObject):
         self._data = ast.literal_eval(self._tmpl_data.render(cert=self))
 
         """Ensure raw file contents in public key, Jinja2 fails to parse if there are CR LF"""
-        self._body['cert_body']['public'] = self._file
-        self._data['cert_body']['public'] = self._file
+        self.public = self._file
 
         """Match the objects common name to the true common name from the certificate and
-        swap out '*' astrix for the keyword wildcard
+        swap out '*' astrix for the keyword wildcard.
+        If the certificate does not have a common name then use the user provided name
         """
-        self._common_name = self.valid_name(self._data['cert_info']['subject']['commonName'])
-        self.name = self.valid_name(self._common_name)
+        if 'commonName' in self._data['cert_info']['subject'] \
+                and len(self._data['cert_info']['subject']['commonName']) > 0:
+            self._common_name = self._data['cert_info']['subject']['commonName']
+        else:
+            logger.warning(f'Certificate {self.name} does not have a value for common name.')
 
-        """Ensure path is the inverse of the true cert common name"""
-        self.path = self.store_path()
-
-        self.type = Cert.PEM.name
+    def policy(self) -> str:
+        self._policy = self._tmpl_policy.render(cert=self)
+        return self._policy
 
     def load(self, pub: str, key: str, certtype: enum.Enum = PEM, chain: str = None) -> None:
         """Read in components of a certificate, given filename paths for each
@@ -119,16 +135,60 @@ class Cert(StoreObject):
                 self._body['cert_body']['chain'] = chain_fp.read()
 
         self._data['cert_body'] = self._body['cert_body']
+        self._data['cert_policy'] = self.policy()
+
+        if not validators.domain(self.name):
+            logger.info(f'{self.name} appears to be a client/server certificate not a web/https certificate')
 
     @classmethod
     def valid_name(cls, value: str) -> str:
         """Some engines might have problems with astrix, as they are used for glob searching and or RBAC.
         Replace it with the key word 'wildcard'. This does not affect the actual certificate."""
-        return value.replace('*', 'wildcard')
+        name = value.replace('*', 'wildcard')
+        if validators.domain(name):
+            return name
+        else:
+            return value
+
+    @property
+    def mount(self) -> str:
+        if validators.domain(self._common_name):
+            self._mount = f"{self._settings['KNOX_VAULT_MOUNT']}/"
+        else:
+            self._mount = f"{self._settings['KNOX_VAULT_MOUNT']}/client"
+        return self._mount
+
+    @property
+    def policy_mount(self) -> str:
+        if validators.domain(self._common_name.replace('*', 'wildcard')):
+            self._mount = f"{self._settings['KNOX_VAULT_MOUNT']}/data"
+        else:
+            self._mount = f"{self._settings['KNOX_VAULT_MOUNT']}/data/client"
+        return self._mount
 
     def subject(self) -> str:
         """Return the certificate subject details"""
-        return json.dumps({attr.oid._name: attr.value for attr in self._x509.subject}, indent=8)
+        subj = {attr.oid._name: attr.value for attr in self._x509.subject}
+        subj['alternativeNames'] = self.subjectaltnames()
+        return json.dumps(subj, indent=8)
+
+    def subjectaltnames(self) -> str:
+        """Return Subject alternate names"""
+        try:
+            cert = self._x509
+            ext = cert.extensions.get_extension_for_oid(x509.ExtensionOID.SUBJECT_ALTERNATIVE_NAME)
+            return json.dumps(f'{ext.value.get_values_for_type(x509.DNSName)}')
+        except Exception as ex:  # noqa E722
+            return ''
+
+    @property
+    def type(self) -> str:
+        return self._type
+
+    @type.setter
+    def type(self, value: str) -> None:
+        if Cert.CertTypes.valid(value):
+            self._type = value
 
     def issuer(self) -> str:
         """Return the certificate issuer details"""
@@ -164,6 +224,13 @@ class Cert(StoreObject):
             'key': key_info
         }, indent=8)
 
+    def isValid(self) -> bool:
+        """Check certificate validity period"""
+        logger.trace(f'Is the date within the validity dates?\n\tNot valid after: {self._x509.not_valid_after}'
+                     f'\n\tNow: {datetime.datetime.now()}'
+                     f'\n\tNot valid before {self._x509.not_valid_before}')
+        return self._x509.not_valid_after > datetime.datetime.now() > self._x509.not_valid_before
+
     @staticmethod
     def to_store_path(common_name: str) -> str:
         """Generate a backend store path based on the certificates common name
@@ -172,14 +239,25 @@ class Cert(StoreObject):
             :return: str
         """
         domainsplit = common_name.split('.')
-        return "/"+"/".join(reversed(domainsplit))
+        return "/" + "/".join(reversed(domainsplit))
 
-    def store_path(self) -> str:
-        return self.to_store_path(self._common_name)
+    @property
+    def name(self) -> str:
+        self._name = self.valid_name(self._common_name)
+        return self._name
+
+    @property
+    def path(self) -> str:
+        if validators.domain(self.name):
+            self._path = Cert.to_store_path(self.name)
+        else:
+            self._path = ''
+        return self._path
 
     def __str__(self) -> str:
         return json.dumps(self._data, indent=4)
 
+    @property
     def private(self) -> str:
         """Unless its a dict, its not loaded yet"""
         if isinstance(self._body, dict):
@@ -187,6 +265,7 @@ class Cert(StoreObject):
         else:
             return ""
 
+    @property
     def chain(self) -> str:
         """Unless its a dict, its not loaded yet"""
         if isinstance(self._body, dict):
@@ -194,9 +273,25 @@ class Cert(StoreObject):
         else:
             return ""
 
+    @chain.setter
+    def chain(self, value) -> None:
+        if isinstance(self._body, dict):
+            self._body['cert_body']['chain'] = value
+            self._data['cert_body']['chain'] = value
+
+    @property
     def public(self) -> str:
         """Convenience method for Jinja2 templates. Jinja2 does not process the string if it has carriage returns."""
-        return self._x509.public_bytes(Encoding.PEM).decode('utf-8').replace('\n', '')
+        if self.type == Cert.PEM.name:
+            return self._x509.public_bytes(Encoding.PEM).decode('utf-8').replace('\n', '')
+        else:
+            raise CertUnsupportedTypeException(type=self.type)
+
+    @public.setter
+    def public(self, value: str) -> None:
+        if isinstance(self._body, dict):
+            self._body['cert_body']['public'] = value
+            self._data['cert_body']['public'] = value
 
     def info(self) -> str:
         return json.dumps(self._info['cert_info'], indent=4)
@@ -217,3 +312,14 @@ class Cert(StoreObject):
             self.load(pub=certfile, key=privkey, chain=chainfile, certtype=Cert.PEM.name)
         except Exception:
             logger.error(f'Failed to generate certificate {self._common_name}')
+
+
+class CertUnsupportedTypeException(Exception):
+    """Exception raised for an unrecognized certificate type"""
+
+    def __init__(self, type: str = None) -> None:
+        self.type = type
+        super().__init__()
+
+    def __str__(self):
+        return f'Unsupported certificate type: {self.type}'
