@@ -40,14 +40,16 @@ from .knox import Knox
               show_default=True,
               help="Sets the level of logging displayed")
 @click.option("--verbose", "-v", is_flag=True, default=False, help="Display log output to console")
+@click.option("--admin", is_flag=True, default=False, help="Enable admin actions", hidden=True)
 @click.version_option(version=pkg_resources.get_distribution('knox').version)
 @click.pass_context
 @logger.catch()
-def cli(ctx, verbose: bool = False, log: str = 'INFO'):
+def cli(ctx, verbose: bool = False, log: str = 'INFO', admin: bool = False):
     """Utilities for managing and storing TLS certificates using backing store (Vault)."""
     ctx.ensure_object(dict)
     ctx.obj['VERBOSE'] = verbose
     ctx.obj['LOG_LEVEL'] = log
+    ctx.obj['ADMIN_MODE'] = admin
     logger.remove()
     if verbose:
         logger.add(sys.stdout,
@@ -85,11 +87,11 @@ def cert(ctx, pub: str, key: str, type: str = 'PEM', chain: str = None):
     ctx.obj['CERT_TYPE'] = type.upper()
 
 
-@cert.command(no_args_is_help=True)
+@cert.command(name="save", no_args_is_help=True)
 @click.argument("name")
 @click.pass_context
 @logger.catch()
-def save(ctx, name):
+def cert_save(ctx, name):
     """Store an existing certificate
     """
     ctx.obj['CERT_NAME'] = name
@@ -98,50 +100,80 @@ def save(ctx, name):
     chain = ctx.obj['CERT_CHAIN']
     certtype = ctx.obj['CERT_TYPE']
 
-    knox = Knox(ctx.obj['LOG_LEVEL'])
+    knox = Knox(ctx)
     certificate = Cert(knox.settings, common_name=name)
     certificate.load(pub=pub,
                      key=key,
                      chain=chain,
                      certtype=certtype)
-    knox.store.save(certificate)
+    if certificate.isValid():
+        knox.store.save(certificate)
+    else:
+        logger.error(f'{certificate.name} is invalid. Check validity Dates:\n {certificate.info()}')
+        sys.exit(2)
 
 
-@cert.command(no_args_is_help=True)
+@cert.command(name="get", no_args_is_help=True)
 @click.argument("name")
 @click.pass_context
 @logger.catch()
-def get(ctx, name):
-    """Retrieve an existing certificate for a given common name from the store
+def cert_get(ctx, name):
+    """Retrieve an existing certificate for a given common name
     """
     ctx.obj['CERT_NAME'] = name
-    knox = Knox(ctx.obj['LOG_LEVEL'])
+    knox = Knox(ctx)
     certificate = Cert(knox.settings, common_name=name)
     certificate.type = ctx.obj['CERT_TYPE']
-    certificate = knox.store.get(certificate.store_path(), name=name, type=certificate.type)
-    with open(certificate.name+"-pub.pem", "w") as pubf:
+    certificate = knox.store.get(certificate.path, name=name, type=certificate.type)
+    with open(f'{certificate.name}-pub.pem', "w") as pubf:
         pubf.write(certificate.body['public'])
-    with open(certificate.name+"-key.pem", "w") as keyf:
+    with open(f'{certificate.name}-key.pem', "w") as keyf:
         keyf.write(certificate.body['private'])
-    with open(certificate.name+"-chain.pem", "w") as chainf:
+    with open(f'{certificate.name}-chain.pem', "w") as chainf:
         chainf.write(certificate.body['chain'])
 
 
-@cert.command(no_args_is_help=True)
+@cert.command(name="gen", no_args_is_help=True)
 @click.argument("name")
 @click.pass_context
 @logger.catch()
-def gen(ctx, name):
+def cert_gen(ctx, name):
     """Create and store a new certificate for a given common name
     """
     ctx.obj['CERT_NAME'] = name
 
-    knox = Knox(ctx.obj['LOG_LEVEL'])
+    knox = Knox(ctx)
     certificate = Cert(knox.settings, common_name=name)
     certificate.generate()
     knox.store.save(certificate)
 
 
+@cert.command(name="aws", no_args_is_help=True)
+@click.argument("name")
+@click.option("--region", default=None, help="Default AWS region")
+@click.option("--profile", default=None, help="Default AWS profile to use")
+@click.pass_context
+@logger.catch()
+def cert_aws(ctx, name, region, profile):
+    """Store a certificate for a given common name in AWS
+    """
+
+    ctx.obj['CERT_NAME'] = name
+    pub = ctx.obj['CERT_PUB']
+    key = ctx.obj['CERT_KEY']
+    chain = ctx.obj['CERT_CHAIN']
+    certtype = ctx.obj['CERT_TYPE']
+
+    knox = Knox(ctx)
+    certificate = Cert(knox.settings, common_name=name)
+    certificate.load(pub=pub, key=key, chain=chain, certtype=certtype)
+    knox.attach("aws")
+    # Save to the attached AWS ACM store
+    knox.stores("aws").save(certificate)
+    # Save to the default store
+    knox.store.save(certificate)
+
+
 @cli.group(no_args_is_help=True)
 @click.pass_context
 def store(ctx) -> dict:
@@ -149,7 +181,7 @@ def store(ctx) -> dict:
     pass
 
 
-@store.command(no_args_is_help=True)
+@store.command(name="find", no_args_is_help=True)
 @click.option("--file", "-f", help="Output file, default stdout")
 @click.option("--output", "-o",
               type=click.Choice(['JSON', 'CSV'], case_sensitive=False),
@@ -166,27 +198,31 @@ def find(ctx, name, file: str = 'stdout', output: str = 'JSON') -> dict:
     i.e. www.example.com becomes /com/example/www/www.example.com when stored.
     Supports wild cards. *.example.com
 
+    To list all certs you might have to escape astrix i.e.
+
+      knox store find \*
+
+    Otherwise your shell may try and interpret it and not pass it to python
     """
     ctx.obj['STORE_FIND_NAME'] = name
     ctx.obj['STORE_FIND_OUTPUT'] = output
     ctx.obj['STORE_FIND_OUTFILE'] = file
-    knox = Knox(ctx.obj['LOG_LEVEL'])
-    if name:
-        results = knox.store.find(pattern=name)  # noqa F841
-        handle = open(file, 'w') if file else sys.stdout
-        if output == 'JSON':
-            handle.write(json.dumps(results, indent=4))
-        else:
-            csv_writer = csv.writer(handle)
-            count = 0
-            for rec in results:
-                if count == 0:
-                    header = rec.keys()
-                    csv_writer.writerow(header)
-                    count += 1
-                csv_writer.writerow(rec.values())
+    knox = Knox(ctx)
+    results = knox.store.find(pattern=name)  # noqa F841
+    handle = open(file, 'w') if file else sys.stdout
+    if output == 'JSON':
+        handle.write(json.dumps(results, indent=4))
+    else:
+        csv_writer = csv.writer(handle)
+        count = 0
+        for rec in results:
+            if count == 0:
+                header = rec.keys()
+                csv_writer.writerow(header)
+                count += 1
+            csv_writer.writerow(rec.values())
 
-        handle.close()
+    handle.close()
 
     return ctx
 
