@@ -22,13 +22,16 @@ class Knox:
     """Composite class for Knox package"""
     _conf: config.Conf
     _store: backend.Store
-    _cli: dict
+    _stores: dict
+    _ctx: dict
 
-    def __init__(self, loglevel=None) -> None:
+    def __init__(self, ctx: dict) -> None:
         """Constructor for Knox """
-
-        self._conf = config.Conf(loglevel)
+        self._ctx = ctx
+        self._conf = config.Conf(ctx.obj['LOG_LEVEL'])
+        self._conf.settings.set('CTX', ctx)
         self._store = backend.Store(self._conf.settings)
+        self._stores = {}
 
     @property
     def settings(self) -> config.Conf.settings:
@@ -42,5 +45,13 @@ class Knox:
 
     @property
     def store(self) -> backend.Store:
-        """Access to the instantiated store engine"""
+        """Access to the default store engine"""
         return self._store
+
+    def stores(self, engine_name: str = None) -> backend.Store:
+        """Retrieve a named store, otherwise the default store"""
+        return self._stores[engine_name] if engine_name is not None else self._store
+
+    def attach(self, engine_name: str) -> bool:
+        """Instantiate an additional store"""
+        self._stores[engine_name] = backend.Store(self._conf.settings, engine_name)
